use game::Game;

pub type SessionToken = [u8; 8];
pub type SessionSecret = [u8; 8];

#[derive(Clone)]
pub struct Player {
    pub user:u32,
    pub connections:Vec<u32>,
}

pub struct Session {
    pub id:u32,
    pub token:SessionToken,
    pub secret:SessionSecret,

    pub game:Game,

    pub p_dawn:Player,
    pub p_dusk:Player,
    pub connections:Vec<u32>,

    pub time:u64,
    pub chain_id:usize,

    pub undo:Option<u8>,
}
impl Session {
    pub fn get_connections(&self) -> Vec<(u32, u8)>
    {
        let mut result = Vec::new();
        for conn in &self.p_dawn.connections {
            result.push((*conn, 0));
        }
        for conn in &self.p_dusk.connections {
            result.push((*conn, 1));
        }
        for conn in &self.connections {
            result.push((*conn, 2));
        }
        result
    }

    pub fn add_connection(&mut self, source:u8, id:u32)
    {
        let connections = match source {
            0 => &mut self.p_dawn.connections,
            1 => &mut self.p_dusk.connections,
            _ => &mut self.connections,
        };

        for i in 0..connections.len() {
            if connections[i] == id {
                connections.remove(i);
                break;
            }
        }

        connections.push(id);
    }

    pub fn remove_connection(&mut self, source:u8, id:u32)
    {
        let connections = match source {
            0 => &mut self.p_dawn.connections,
            1 => &mut self.p_dusk.connections,
            _ => &mut self.connections,
        };

        for i in 0..connections.len() {
            if connections[i] == id {
                connections.remove(i);
                break;
            }
        }
    }

    pub fn spectators(&self) -> usize
    {
        self.connections.len()
    }
}
