use crate::{
    app::session::SessionToken,
    util::pack::{pack_u16, pack_u32},
};

use game::history::Play;

use super::Packet;

#[derive(Clone)]
pub struct PacketGameState {
    pub token:SessionToken,
}
impl PacketGameState {
    pub fn new() -> Self
    {
        Self {
            token:SessionToken::default(),
        }
    }
}
impl Packet for PacketGameState {
    type Data = Self;

    fn decode(data:&Vec<u8>, index:&mut usize) -> Result<Self::Data, ()>
    {
        let mut result = Self::new();

        if data.len() - *index == 8 {
            for i in 0..8 {
                result.token[i] = data[*index];
                *index += 1;
            }
            Ok(result)
        } else {
            Err(())
        }
    }
}

#[derive(Clone)]
pub struct PacketGameStateResponse {
    pub status:u16,
    pub token:SessionToken,
    pub player:u8,
    pub dawn_handle:String,
    pub dusk_handle:String,
    pub dawn_online:bool,
    pub dusk_online:bool,
    pub spectators:u32,
    pub history:Vec<Play>,
}
impl PacketGameStateResponse {
    pub fn new() -> Self
    {
        Self {
            status:0,
            token:SessionToken::default(),
            player:2,
            dawn_handle:String::new(),
            dusk_handle:String::new(),
            dawn_online:false,
            dusk_online:false,
            spectators:0,
            history:Vec::new(),
        }
    }
}
impl Packet for PacketGameStateResponse {
    type Data = Self;

    fn encode(&self) -> Vec<u8>
    {
        let mut history_bytes = Vec::new();
        for play in &self.history {
            let mut data = 0;
            data |= play.source as u16;
            data |= (play.from as u16) << 4;
            data |= (play.to as u16) << 10;
            history_bytes.append(&mut pack_u16(data));
        }

        let mut flags = 0u16;
        flags |= self.player as u16;
        flags |= (self.dawn_online as u16) << 2;
        flags |= (self.dusk_online as u16) << 3;
        
        [
            pack_u16(self.status),
            self.token.to_vec(),
            pack_u16(flags),
            pack_u32(self.spectators),
            pack_u16(self.dawn_handle.len() as u16),
            self.dawn_handle.as_bytes().to_vec(),
            pack_u16(self.dusk_handle.len() as u16),
            self.dusk_handle.as_bytes().to_vec(),
            pack_u16(self.history.len() as u16),
            history_bytes,
        ].concat()
    }
}
